# database.py
import psycopg2
from psycopg2 import sql
from decouple import config
from bypasssafe.account import Account
import bcrypt


class Database:
    """
    Classe para interação com o banco de dados.

    Esta classe contém métodos para salvar contas, buscar contas associadas a um mestre,
    autenticar usuários e obter informações de uma conta por nome de usuário.
    """

    DATABASE_URL = config("DATABASE_URL")
    
    @staticmethod
    def save_account(account):
        try:
            connection = psycopg2.connect(Database.DATABASE_URL)
            cursor = connection.cursor()
            
            master_id = int(account.master_id[0])  # Extract the first element from the tuple

            # Usando parâmetros preparados para evitar injeção SQL
            create_account_query = sql.SQL(
                "INSERT INTO accounts (master_id, username, password, email) VALUES ({}, {}, {}, {})"
            ).format(
                sql.Literal(master_id),
                sql.Literal(account.username),
                sql.Literal(account.password),
                sql.Literal(account.email),
            )
            
            cursor.execute(create_account_query)
            connection.commit()

            print("Conta salva com sucesso.")

        except Exception as e:
            print(f"Erro ao salvar conta: {e}")

        finally:
            cursor.close()
            connection.close()
            
    @staticmethod
    def save_master_account(master_account):
        try:
            connection = psycopg2.connect(Database.DATABASE_URL)
            cursor = connection.cursor()

            create_master_account_query = sql.SQL(
                "INSERT INTO masters (username, email, password) VALUES ({}, {}, {})"
            ).format(
                sql.Literal(master_account.username),
                sql.Literal(master_account.email),
                sql.Literal(master_account.password),
            )

            cursor.execute(create_master_account_query)
            connection.commit()

            print("Conta mestre salva com sucesso.")

        except Exception as e:
            print(f"Erro ao salvar conta mestre: {e}")

        finally:
            cursor.close()
            connection.close()

    @staticmethod
    def get_accounts_by_master_id(master_id):
        """
        Obtém todas as contas associadas a um mestre.

        Args:
            master_id (int): O ID do mestre para o qual as contas estão associadas.

        Returns:
            list: Uma lista de tuples representando as contas associadas ao mestre.
        """
        connection = psycopg2.connect(Database.DATABASE_URL)
        cursor = connection.cursor()

        select_accounts_query = "SELECT * FROM accounts WHERE master_id = %s"
        cursor.execute(select_accounts_query, (master_id,))
        accounts = cursor.fetchall()

        cursor.close()
        connection.close()

        return accounts

    @staticmethod
    def authenticate_user(email, password):
        try:
            connection = psycopg2.connect(Database.DATABASE_URL)
            cursor = connection.cursor()

            # Consulta parametrizada para obter o id do mestre e a senha codificada
            select_master_query = sql.SQL("SELECT id, password FROM masters WHERE email = {}").format(
                sql.Literal(email)
            )

            cursor.execute(select_master_query)
            result = cursor.fetchone()

            if result and bcrypt.checkpw(password.encode(), result[1].encode()):
                # Se a senha estiver correta, retorna o id do mestre
                return result[0]
            else:
                # Senha incorreta ou usuário não encontrado
                return None

        except Exception as e:
            print(f"Um erro ocorreu durante a autenticação do usuário: {e}")
            return None

        finally:
            cursor.close()
            connection.close()
        
    @staticmethod
    def get_account_by_username(master_id, username):
        """
        Obtém informações de uma conta por nome de usuário.

        Args:
            master_id (int): O ID do mestre ao qual a conta está associada.
            username (str): O nome de usuário da conta.

        Returns:
            Account or None: A conta correspondente se encontrada, None caso contrário.
        """
        connection = psycopg2.connect(Database.DATABASE_URL)
        cursor = connection.cursor()

        select_account_query = (
            "SELECT * FROM accounts WHERE master_id = %s AND username = %s"
        )
        cursor.execute(select_account_query, (master_id, username))
        account_data = cursor.fetchone()

        cursor.close()
        connection.close()

        if account_data:
            account = Account(
                account_data[1], account_data[2], account_data[3], account_data[4]
            )
            return account
        else:
            return None



# main.py
from bypasssafe.account import MasterAccount, Account
from bypasssafe.database import Database
from bypasssafe.pass_gen import PasswordGenerator
import os
import sys
import time
import pathlib as Path
import getpass
import os

"""
Adicionando o diretório raiz do projeto ao path do Python.
"""
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

def login():
    email = input("Email: ")
    password = getpass.getpass("Senha Mestre: ")

    try:
        master_id = Database.authenticate_user(email, password)
        if master_id is not None:
            master_id = int(master_id)  # Convert to an integer
            print("Login realizado com sucesso.")
            return master_id, email
        else:
            print("Falha no Login. Email ou senha inválido.")
            return None, None
    except Exception as e:
        print(f"Um erro ocorreu durante o login: {e}")
        return None, None

def create_master_account():
    """
    Cria uma conta mestre no sistema.

    Solicita informações do usuário, cria uma conta mestre e a salva no banco de dados.
    """
    username = input("Digite seu nome: ")
    email = input("Digite seu Email: ")
    password = getpass.getpass("Digite sua senha mestra: ")

    master_account = MasterAccount(username, password, email)

    try:
        if Database.save_master_account(master_account):
            print("Falha ao criar a conta.")
        else:
            print("Conta Criada com sucesso! Faça login já.")
    except Exception as e:
        print(f"ERRO: {e}")

def create_account(master_id):
    """
    Cria uma nova conta associada à conta mestre fornecida.

    Solicita informações do usuário, cria uma conta e a salva no banco de dados.

    Args:
        master_id (int): O ID da conta mestre à qual a nova conta será associada.
    """
    username = input("Digite o username: ")
    email = input("Digite o email: ")
    choice = input("Deseja digitar ou gerar uma senha? (1 - Digitar / 2 - Gerar): ")

    if choice == "1":
        password = getpass.getpass("Digite a senha: ")
    elif choice == "2":
        password = generate_password()
    else:
        print("Por favor, digite uma opção válida. (1 ou 2)")
        return

    account = Account(master_id, username, password, email)
    Database.save_account(account)

def generate_password():
    """
    Gera uma senha aleatória com base nas preferências do usuário.

    Retorna:
        str: A senha gerada.
    """
    password_generator = PasswordGenerator()
    length = int(input("Qual o tamanho da senha: "))
    digits = int(input("Digite a quantidade de números: "))
    symbols = int(input("Digite a quantidade de símbolos: "))
    uppercase = (input("Deseja misturar letras maiúsculas com minúsculas? (S/N): ").lower() == "S")

    password = password_generator.generate_password(length, digits, symbols, uppercase)
    print(f"Senha Gerada: {password}")
    return password

def search_account(account, username_to_search):
    """
    Procura e exibe informações sobre uma conta associada à conta mestre.

    Args:
        master_id (int): O ID da conta mestre à qual a conta a ser pesquisada está associada.
        username_to_search (str): O nome de usuário da conta a ser pesquisada.
    """
    try:
        account = Database.get_account_by_username(account[0], username_to_search)
        if account.username == username_to_search:
            print(f"Informações da Conta:")
            print(f"Nome: {account.username}")
            print(f"Email: {account.email}")
            print(f"Senha: {account.password}")
            input("Pressione Enter para voltar ao menu...")
        else:
            print(f"Conta com nome '{username_to_search}' não encontrada.")
            input("Pressione Enter para voltar ao menu...")
    except Exception as e:
        print(f"Erro durante a pesquisa: {e}")
        input("Pressione Enter para voltar ao menu...")

def clear_console():
    """
    Clears the console screen.
    """
    os.system('cls' if os.name == 'nt' else 'clear')

def register_script():
    # Obter o caminho do script atual
    script_path = Path(__file__).resolve()

    # Obter o caminho do diretório de scripts local
    scripts_dir = Path.home() / ".local" / "bin"

    # Criar o diretório se não existir
    scripts_dir.mkdir(parents=True, exist_ok=True)

    # Criar um link simbólico para o script no diretório de scripts
    link_path = scripts_dir / script_path.name
    link_path.symlink_to(script_path)

    print(f"Script registrado em {link_path}")



from bypasssafe.main import create_master_account, login, create_account, generate_password, search_account,  clear_console
import time

def main():
    """
    Função principal do programa.

    Controla o fluxo principal do programa, incluindo a exibição de menus, chamadas de funções e interações com o usuário.
    """
    master_id = None

    while master_id is None:
        clear_console()
        print("1 - Fazer Login")
        print("2 - Criar Conta Master")
        ja = int(input("Escolha uma opção: "))
        if ja == 1:
            master_id = login()
        elif ja == 2:
            create_master_account()
        else:
            print("Opção inválida. Por favor, digite um número entre 1 e 2.")

    if master_id is not None:
        while True:
            clear_console()
            print("Welcome!")
            print("\nMenu:")
            print("1. Cadastrar Conta")
            print("2. Gerar Senha")
            print("3. Procurar Conta")
            print("4. Sair")

            choice = input("Enter your choice (1-4): ")

            if choice == "1":
                create_account(master_id)
                input("Pressione Enter para voltar ao menu...")
                time.sleep(2)  # Aguarda 2 segundos antes de limpar o console e mostrar o próximo menu
            elif choice == "2":
                generate_password()
                time.sleep(2)
            elif choice == "3":
                username_to_search = input("Digite o nome da conta que deseja buscar: ")
                search_account(master_id, username_to_search)
                input("Pressione Enter para voltar ao menu...")
                time.sleep(2)
            elif choice == "4":
                print("Fechando. Até mais!")
                break
            else:
                print("Escolha inválida. Por favor, digite um número entre 1 e 4.")
                time.sleep(2)


if __name__ == "__main__":
    main()


# pass_gen.py
import random
import string

class PasswordGenerator:
    """
    Classe para gerar senhas aleatórias com base em requisitos específicos.

    A classe fornece um método estático para gerar senhas que podem incluir letras,
    números e símbolos, com a opção de incluir ou não letras maiúsculas.
    """

    @staticmethod
    def generate_password(length, num_digits, num_symbols, mix_case):
        """
        Gera uma senha aleatória com base nos parâmetros fornecidos.

        Args:
            length (int): O comprimento total da senha.
            num_digits (int): O número de dígitos na senha.
            num_symbols (int): O número de símbolos na senha.
            mix_case (bool): Indica se a senha deve incluir letras maiúsculas.

        Returns:
            str: A senha gerada.
        """
        all_chars = string.ascii_letters + string.digits + string.punctuation
        if not mix_case:
            all_chars = string.ascii_lowercase + string.digits + string.punctuation

        digits = "".join(random.choice(string.digits) for _ in range(num_digits))
        symbols = "".join(random.choice(string.punctuation) for _ in range(num_symbols))
        letters = "".join(
            random.choice(all_chars) for _ in range(length - num_digits - num_symbols)
        )

        password = "".join(random.sample(digits + symbols + letters, length))
        return password



# account.py

class MasterAccount:
    """
    Representa uma conta mestre no sistema.

    Atributos:
        username (str): O nome de usuário associado à conta mestre.
        password (str): A senha mestra, armazenada como um hash.
        email (str): O endereço de email associado à conta mestre.
    """

    def __init__(self, username, hashed_password, email):
        """
        Inicializa uma nova instância de MasterAccount.

        Args:
            username (str): O nome de usuário associado à conta mestre.
            hashed_password (str): A senha mestra, armazenada como um hash.
            email (str): O endereço de email associado à conta mestre.
        """
        self.username = username
        self.password = hashed_password  
        self.email = email


class Account:
    """
    Representa uma conta associada a uma conta mestre no sistema.

    Atributos:
        master_id (int): O ID da conta mestre à qual esta conta está associada.
        username (str): O nome de usuário associado à conta.
        password (str): A senha associada à conta.
        email (str): O endereço de email associado à conta.
    """

    def __init__(self, master_id, username, password, email):
        """
        Inicializa uma nova instância de Account.

        Args:
            master_id (int): O ID da conta mestre à qual esta conta está associada.
            username (str): O nome de usuário associado à conta.
            password (str): A senha associada à conta.
            email (str): O endereço de email associado à conta.
        """
        self.master_id = master_id
        self.username = username
        self.password = password
        self.email = email
